@@ -40,6 +40,8 @@ import org.apache.pulsar.client.api.Message;
 import org.apache.pulsar.client.api.MessageId;
 import org.apache.pulsar.client.api.PulsarClient;
 import org.apache.pulsar.client.api.PulsarClientException;
+import org.apache.pulsar.client.api.PulsarClientException.AlreadyClosedException;
+import org.apache.pulsar.client.api.RedeliveryBackoff;
 import org.apache.pulsar.client.api.RegexSubscriptionMode;
 import org.apache.pulsar.client.api.Schema;
 import org.apache.pulsar.client.api.SubscriptionInitialPosition;
@@ -90,6 +92,8 @@ class AdaptedReactiveMessageConsumerTests {
 			.maxRedeliverCount(1)
 			.build();
 
+		TestRedeliveryBackoff testRedeliveryBackoff = new TestRedeliveryBackoff();
+
 		ConsumerConfigurationData<String> expectedConsumerConf = new ConsumerConfigurationData<>();
 		expectedConsumerConf.setTopicNames(new HashSet<>(Arrays.asList("my-topic", "my-rlt")));
 		expectedConsumerConf.setSubscriptionName("my-sub");
@@ -115,6 +119,8 @@ class AdaptedReactiveMessageConsumerTests {
 		expectedConsumerConf.setTickDurationMillis(TimeUnit.SECONDS.toMillis(4));
 		expectedConsumerConf.setAcknowledgementsGroupTimeMicros(TimeUnit.SECONDS.toMicros(5));
 		expectedConsumerConf.setNegativeAckRedeliveryDelayMicros(TimeUnit.SECONDS.toMicros(6));
+		expectedConsumerConf.setNegativeAckRedeliveryBackoff(testRedeliveryBackoff);
+		expectedConsumerConf.setAckTimeoutRedeliveryBackoff(testRedeliveryBackoff);
 		expectedConsumerConf.setDeadLetterPolicy(deadLetterPolicy);
 		expectedConsumerConf.setRetryEnable(true);
 		expectedConsumerConf.setReceiverQueueSize(7);
@@ -151,6 +157,8 @@ class AdaptedReactiveMessageConsumerTests {
 			.ackTimeoutTickTime(Duration.ofSeconds(4))
 			.acknowledgementsGroupTime(Duration.ofSeconds(5))
 			.negativeAckRedeliveryDelay(Duration.ofSeconds(6))
+			.negativeAckRedeliveryBackoff(testRedeliveryBackoff)
+			.ackTimeoutRedeliveryBackoff(testRedeliveryBackoff)
 			.deadLetterPolicy(deadLetterPolicy)
 			.retryLetterTopicEnable(true)
 			.receiverQueueSize(7)
@@ -406,4 +414,35 @@ class AdaptedReactiveMessageConsumerTests {
 			.verifyError(PulsarClientException.InvalidMessageException.class);
 	}
 
+	@Test
+	void closeConsumerExceptionIsIgnored() throws Exception {
+		PulsarClientImpl pulsarClient = spy(
+				(PulsarClientImpl) PulsarClient.builder().serviceUrl("http://dummy").build());
+
+		Consumer<String> consumer = mock(Consumer.class);
+		doReturn(CompletableFuture.failedFuture(new AlreadyClosedException("Already closed"))).when(consumer)
+			.closeAsync();
+
+		doReturn(CompletableFuture.completedFuture(consumer)).when(pulsarClient)
+			.subscribeAsync(any(ConsumerConfigurationData.class), eq(Schema.STRING), isNull());
+
+		ReactiveMessageConsumer<String> reactiveConsumer = AdaptedReactivePulsarClientFactory.create(pulsarClient)
+			.messageConsumer(Schema.STRING)
+			.topic("my-topic")
+			.subscriptionName("my-sub")
+			.build();
+
+		StepVerifier.create(reactiveConsumer.consumeNothing()).expectComplete().verify();
+		verify(consumer).closeAsync();
+	}
+
+	static class TestRedeliveryBackoff implements RedeliveryBackoff {
+
+		@Override
+		public long next(int redeliveryCount) {
+			return redeliveryCount * 2L;
+		}
+
+	}
+
 }
